%  This file shows how to run the MUDICA
%  analyses  decribed in 
%   Williams, L.J., Abdi, H.,  French, R., & Orange, J.B.
%    (in press, 2010).
%    A tutorial on Multi-Block Discriminant Correspondence Analysis 
%    (MUDICA): A new method for
%    analyzing discourse data from clinical populations.
%    Journal of Speech Language and Hearing Research.
%  Computational details are given in Appendix C of this paper.
%   This script shows and analyzes the data for MUDICA on 
%   Trouble Source Repair in 
%   Conversations between individuals 
%   with Alzehimer Disease and their Spouses 
%   (See Williams et al., 2010 for details).
%
%  History
%  First version:
%   Herve Abdi  
%               November 13, 2008,
%    Revisions: November 18, 2008,    
%               July 2009 to include tolerance intervals
%               September 2009 to include prediction intervals
%               Final cleaning February 5 to 20, 2010.
%   Current version: February 21, 2010.
%   Herve Abdi
%   herve@utdallas.edu
%   www.utdallas.edu/~herve
%
% Functions (m-files) used by this program
%  (these functions need to be in the active
%   matlab path, e.g. in the same directory 
%   as this file): 
% 1. Computational Routines:
%    dummy_of, DCA_pred, BootDCABal, MultiBlockDICA,
%    R2PermOnFactors
% 2. Graph Routines:
%    PlotDCAConfTolePred_Ellipses, plotxyha, PlotUglyBlocks
%    PlotEllipse, plotxyplain
%
% ************************************************************
% Here we call a version of DCA_pred
% which takes care of the columns with only one non-zero entry
% A column with only one-non zero entry will cause problem when
% we jackknife because when the corresponding row is left out in
% the jackknife we have a table with an empty column and this causes
% a division by zero error.
% In the current version of DCA_pred 
%   These values will be droped by the program
% Col2Drop=[8,9, 12, 18, 25, 28, 29, 43, 47, 53];
% Much better way of finding these columns would be
% Col2Drop=find(sum(X~=0)==1);
%


% First clean the workspace and screen
clear % clear the workspace
clc   % clear the screen

disp('Multi-Block Disscriminant Analysis (MUDICA)')
disp('This scripts analyses the data from ')
disp('Willams, Abdi, French & Orange (2010)')
disp('see www.utdallas.edu/~herve for more information')
disp(' ')

% Now get some parameters: title, printing or not etc.
%* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%************************************************************
%*% -> This is your title. 
%*% -> Change it to fit your data
ze_title=['DICA: AD and Spouses EDAT6 removed'];
%*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%*% -> Tis is the generic title for the graph
% 
gen_name='DICA_WAFO_';
%*% imp=0 => we do not want to save the graphs as files
%*% imp=1 => we want to to save the graphs as files
imp=1;
imp=0;
%*% What type of graph? Here postscript
printype='-depsc2';
%*% Extension for the graph file names
%   here '.esp' for eps files
printextension='.eps';



% Now Enter the data.
% We need the data per se (X matrix, group matrix and block matrix)
% Name of the group, rows, and variables
% In this version the names of the blocks/subtables
% are generated by the program
%**********************************************************
% This is the data matrix
% -> Change it for your example 
X=[...
4	2	0	1	0	0	0	0	1	0	0	0	0	2	1	0	3	0	0	4	0	2	0	2	0	0	2	0	1	0	2	2	2	2	0	7	4	1	0	0	1	0	0	0	0	0	1	0	1	4	3	2	0	5	1	0	1	0	0	2	2	2	1	6	1	6	1	0
7	1	1	0	1	1	1	0	0	0	1	1	0	0	3	0	3	0	0	5	0	7	0	0	0	0	1	0	0	0	6	1	6	1	0	6	1	0	1	1	1	0	0	2	0	0	0	0	2	1	1	2	0	6	0	1	0	0	0	5	2	1	0	5	1	4	1	1
5	2	0	2	0	1	0	0	0	0	0	0	0	4	0	3	1	0	0	5	0	6	0	0	0	0	0	0	0	0	4	1	4	1	0	3	2	0	0	0	0	0	0	0	1	0	0	0	2	0	2	0	0	1	1	0	0	0	0	3	0	0	1	2	1	2	1	0
3	1	0	0	2	0	0	0	0	0	0	0	0	3	0	2	1	0	0	3	0	1	1	1	0	0	0	0	0	0	3	0	3	0	0	5	4	0	0	0	0	0	0	0	0	1	0	0	3	1	2	2	0	3	1	0	0	0	0	6	0	0	1	3	2	3	2	0
9	6	0	2	0	0	0	0	0	1	0	0	0	6	2	5	3	0	0	8	0	8	1	0	0	3	0	0	0	0	7	2	6	3	0	24	17	2	0	2	2	0	1	0	0	0	0	0	13	8	7	14	0	23	1	0	0	1	2	11	0	4	10	21	3	18	4	2
3	2	0	1	0	0	0	0	0	0	0	0	0	2	1	2	1	0	0	2	1	4	0	0	0	0	0	0	0	0	2	1	1	2	0	2	1	1	0	0	0	0	0	0	0	0	0	0	1	1	1	1	0	2	0	0	0	0	0	2	0	0	0	2	0	2	0	0
1	1	0	0	0	0	0	0	0	0	0	0	0	0	1	1	0	0	0	1	0	1	0	0	0	0	1	0	0	0	0	1	0	1	0	3	1	0	1	0	0	0	0	0	1	0	0	0	2	0	2	0	0	2	0	0	0	0	0	2	0	0	1	3	0	3	0	0
13	7	2	1	1	0	1	0	0	1	0	0	0	3	8	6	5	0	0	12	0	8	3	4	0	2	5	1	0	0	7	6	6	4	3	16	1	2	1	6	2	1	0	1	0	1	0	1	5	5	8	1	1	14	0	1	0	0	0	11	4	0	1	15	1	14	2	0
5	3	1	0	0	0	0	0	0	0	1	0	0	2	2	2	2	0	0	4	0	4	0	1	0	1	0	0	0	1	4	1	3	1	1	6	2	1	0	0	0	1	0	0	0	2	0	0	2	1	3	0	0	4	0	1	0	0	0	6	1	1	0	4	2	3	2	1
11	6	2	0	0	0	1	0	0	0	1	0	1	2	6	7	1	0	0	8	1	6	1	0	0	6	0	0	0	0	9	2	7	3	1	8	4	2	0	0	1	1	0	0	0	0	0	0	2	4	3	3	0	8	0	1	0	0	0	5	1	0	3	6	2	6	1	1
8	3	0	0	2	2	0	0	0	1	0	0	0	1	4	3	2	0	0	7	0	3	1	3	0	1	2	0	0	0	6	2	6	2	0	7	3	1	0	1	1	0	0	1	0	0	0	0	2	3	4	1	0	7	0	1	0	0	1	6	0	0	1	6	1	5	2	0
22	19	1	0	1	1	0	0	0	0	0	0	0	5	16	14	4	1	2	12	10	2	18	0	1	2	7	0	0	1	17	5	15	7	0	16	11	1	0	0	3	0	0	1	0	0	0	0	6	6	6	6	0	15	1	1	1	0	0	14	3	1	5	10	6	8	5	3
12	5	3	0	2	0	1	1	0	0	0	0	0	6	4	5	5	0	0	10	2	7	3	0	0	5	1	0	0	0	9	3	7	3	2	10	4	2	3	0	1	0	0	0	0	0	0	0	2	7	6	3	0	9	1	2	0	0	0	8	1	0	4	6	4	4	2	4
3	2	0	0	1	0	0	0	0	0	0	0	0	2	1	2	0	0	1	3	0	1	0	2	0	1	1	0	0	0	2	1	1	1	1	2	0	0	0	0	0	0	0	0	2	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0	2	2	0	2	0	0
4	3	0	0	0	0	0	0	0	0	0	0	1	0	3	1	2	0	0	3	0	2	1	1	0	0	0	0	0	0	4	0	4	0	0	7	2	1	0	0	2	0	0	0	2	0	0	0	2	1	3	0	0	5	0	0	0	1	0	6	1	1	0	5	2	5	2	0
3	2	0	0	0	0	0	0	0	1	0	0	0	1	1	1	0	0	1	2	0	2	0	0	0	1	0	0	0	0	3	0	1	1	1	3	2	0	0	0	0	0	0	0	0	0	0	1	0	2	1	1	0	2	0	0	0	0	0	2	0	0	1	3	0	3	0	0
12	6	1	1	1	0	0	0	0	1	2	0	0	3	6	5	4	0	0	8	1	1	0	0	0	11	0	0	0	0	12	0	11	0	1	1	1	0	0	0	0	0	0	0	0	0	0	0	0	1	0	1	0	1	0	0	0	0	0	1	0	0	0	1	0	1	0	0
];
%  get the # of rows and columns %%%%%%%%%%%%%%


[nI,nJ]=size(X);
%**************************************************
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% -> These are your variable names.
% -> Change them to fit your data
% You need as many names as mcX has columns
n=0;
%
n=n+1;nom_c(n)={' P-TS'};
n=n+1;nom_c(n)={' P-Tintr'};
n=n+1;nom_c(n)={' P-Tintr+1'};
n=n+1;nom_c(n)={' P-Tintr+2'};
n=n+1;nom_c(n)={' P-Tintr>2'};
n=n+1;nom_c(n)={' P-STIntr'};
n=n+1;nom_c(n)={' P-STIntr+1'};
n=n+1;nom_c(n)={' P-STIntr+2'};
n=n+1;nom_c(n)={' P-STIntr>2'};
n=n+1;nom_c(n)={' P-SUIntr'};
n=n+1;nom_c(n)={' P-SUIntr+1'};
n=n+1;nom_c(n)={' P-SUIntr+2'};
n=n+1;nom_c(n)={' P-SUIntr>2'};
n=n+1;nom_c(n)={' P-GT-New'};
n=n+1;nom_c(n)={' P-GT-Re'};
n=n+1;nom_c(n)={' P-Ch'};
n=n+1;nom_c(n)={' P-Sh'};
n=n+1;nom_c(n)={' P-NCCh'};
n=n+1;nom_c(n)={' P-TSh'};
n=n+1;nom_c(n)={' P-ST-New'};
n=n+1;nom_c(n)={' P-ST-Re'};
n=n+1;nom_c(n)={' P-Discourse-px'};
n=n+1;nom_c(n)={' P-Cognitive-px'};
n=n+1;nom_c(n)={' Px-Semantic-px'};
n=n+1;nom_c(n)={' P-Phonological-px'};
n=n+1;nom_c(n)={' P-Other-px'};
n=n+1;nom_c(n)={' P-Discourse-sp'};
n=n+1;nom_c(n)={' P-Cognitive-sp'};
n=n+1;nom_c(n)={' P-Semantic-sp'};
n=n+1;nom_c(n)={' P-Other-sp'};
n=n+1;nom_c(n)={' P-Simple'};
n=n+1;nom_c(n)={' P-Complex'};
n=n+1;nom_c(n)={' P-Most-successful'};
n=n+1;nom_c(n)={' P-Successful'};
n=n+1;nom_c(n)={' P-Unsuccessful'};
nJ1=n;
%
n=n+1;nom_c(n)={' S-TS'};
n=n+1;nom_c(n)={' S-Tintr'};
n=n+1;nom_c(n)={' S-Tintr+1'};
n=n+1;nom_c(n)={' S-Tintr+2'};
n=n+1;nom_c(n)={' S-Tintr>2'};
n=n+1;nom_c(n)={' S-STIntr'};
n=n+1;nom_c(n)={' S-STIntr+1'};
n=n+1;nom_c(n)={' S-STIntr+2'};
n=n+1;nom_c(n)={' S-STIntr>2'};
n=n+1;nom_c(n)={' S-SUIntr'};
n=n+1;nom_c(n)={' S-SUIntr+1'};
n=n+1;nom_c(n)={' S-SUIntr+2'};
n=n+1;nom_c(n)={' S-SUIntr>2'};
n=n+1;nom_c(n)={' S-GT-New'};
n=n+1;nom_c(n)={' S-GT-Re'};
n=n+1;nom_c(n)={' S-Ch'};
n=n+1;nom_c(n)={' S-Sh'};
n=n+1;nom_c(n)={' S-NCCh'};
n=n+1;nom_c(n)={' S-ST-New'};
n=n+1;nom_c(n)={' S-ST-Re'};
n=n+1;nom_c(n)={' S-Discourse-px'};
n=n+1;nom_c(n)={' S-Cognitive-px'};
n=n+1;nom_c(n)={' S-Semantic-px'};
n=n+1;nom_c(n)={' S-Other-px'};
n=n+1;nom_c(n)={' S-Discourse-sp'};
n=n+1;nom_c(n)={' S-Cognitive-sp'};
n=n+1;nom_c(n)={' S-Semantic-sp'};
n=n+1;nom_c(n)={' S-Other-sp'};
n=n+1;nom_c(n)={' S-Simple'};
n=n+1;nom_c(n)={' S-Complex'};
n=n+1;nom_c(n)={' S-Most-successful'};
n=n+1;nom_c(n)={' S-Successful'};
n=n+1;nom_c(n)={' S-Unsuccessful'};
nJ2=n-nJ1;

% nJ1 and Nj2 are (respectivley) the
% number of columns corresponding to the first subtable
% which is the data from the participant (i.e., patient)
% and the spouse.
% Here we define the Block Indicator Matrix
% It is an 1*nJ (row) vector
% the value of the j-th element
% gives the block to which this column belongs.
% First define the "Block Indicator vector"
% Here the first nJ1 columns are the participants' variables
%      the next nJ2 columns are the participants' spouse's variables
Tnum=[ones(1,nJ1),2*ones(1,nJ2)];
%
nK=max(Tnum); %nK is number of blocks
% NB: In this version the blocks 
%  will be labelled from 1 to # of blocks in the graphs.

%%% Test the # of colums names
if nJ~=n; 
    erreur(['Error ->  (Wrong # of column names)!']);
end
%***********************************************
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% -> These are your observations names.
% -> Change them to fit your data
m=0;
m=m+1;nom_r{m}={'CTRL1'};
m=m+1;nom_r{m}={'CTRL2'};
m=m+1;nom_r{m}={'CTRL3'};
m=m+1;nom_r{m}={'CTRL4'};
m=m+1;nom_r{m}={'CTRL5'};
m=m+1;nom_r{m}={'CTRL6'};
%
m=m+1;nom_r{m}={'EDAT1'};
m=m+1;nom_r{m}={'EDAT2'};
m=m+1;nom_r{m}={'EDAT3'};
m=m+1;nom_r{m}={'EDAT4'};
m=m+1;nom_r{m}={'EDAT5'};
%
m=m+1;nom_r{m}={'MDAT1'};
m=m+1;nom_r{m}={'MDAT2'};
m=m+1;nom_r{m}={'MDAT3'};
m=m+1;nom_r{m}={'MDAT4'};
m=m+1;nom_r{m}={'MDAT5'};
m=m+1;nom_r{m}={'MDAT6'};
%
if nI~=m; 
    erreur(['Error -> (Wrong # of row names)!']);
end
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% We need the name of the categories/groups of rows
% here we have three categories/groups" of dyads of participants
%  CTRL. EDAT. MDAT
% (Control, Early Dementia, Middle Dementia).
%
nc=0;
nc=nc+1;nom_cat{nc}={'CTRL'};
nc=nc+1;nom_cat{nc}={'EDAT'};
nc=nc+1;nom_cat{nc}={'MDAT'};

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % For each observation we need the number describing its category
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Y=[1 1 1 1 1 1,...
   2 2 2 2 2,...
   3 3 3 3 3 3]';
% Here 1,2, or 3 means that the corresponding row
%  describs an observation (i.e., dyad) coming from the 
% CTRL, EDAT, or MDAT group
[niY,njY]=size(Y);
if niY~=nI;error(['Wrong number of rows for Y!']);end
nc2=max(Y);
if nc2~=nc;error(['Wrong number of categories for Y!']);end
% %%  Done with entering the data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





% 1. First part. Run the DICA 
%    (DIscriminant Correspondance Analysis).
%    Program: DCA_pred

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Create the G vector (need to call DCA_pred)
% which is a nI*nc 0/1 vector where a 1 means the row belong 
% the column category
G=dummy_of(Y);



% 1.1 Run DICA to get fixed and random (jackknifed) coordinates
%     for the observations and confusion matrices.
%     (gives also coordinates for the groups and the columns,
%     see "help DCA_pred" for more information.
% 

[lambda,t,d,fi,ri,ci,dj,fj,rj,cj,...
  f_obs,d_obs,c_obs,Group_Assigned,ConfMat,GA_jack,CM_jack,...
  f_obs_jack,fi_jack,f_obs_pred]=DCA_pred(X,G) ;


% 1.2 Compute confidence intervals for the group/category centers 
%     We use BootDCABal because we want the group sizes to be
%     preserved in the BootStrap
%

% niter=# of bootstraped iterations
niter=1000;
% Here 
% X      is the original data matrix
% G      is the group matrix
% fj     is factors on J set
% lambda is eigenvalue vector from DCA
% niter  is # of iterations for bootstrap
[FBoot]=BootDCABal(X,G,fj,lambda,niter);

%
% 1.3 Compute  R^2 and permutation test for p(R^2)
nperm=1000; % # of permutations
m_obs=sum(X,2)./sum(sum(X)); % masses of the rows, 
%        here we use the plain CA masses for m_obs
[R2,F,P_perm,p_Fth,R2_sim,F_sim,InB,InT]=...
             R2PermOnFactors(f_obs,G,nperm,m_obs);


% ***************************************************
% 2. Go for the "MultiBlock" analysis
% Get the Blocks Coordinates
%
[BlockFactor]=MultiBlockDICA(X,G,Tnum,fj,lambda);


%
disp(['MUDICA done. Going for graphics'])

% 3. Plot the results 
% ********************************************************************
% Call the plot for the ellipses
% 
% NB The current routine is the old "buggedroutine"
% which plot 100% confidence intervals 
% to be fixed later
% HA November 13, 2008 

% First what are the axes that we want to use
% for the plot?
axe_horizontal=1;
axe_vertical=2;
LesAxes=[axe_horizontal,axe_vertical];
%

 nfig=0; %initialize for PlotDCAEllipses
% Here we print the 
% confidence, tolerance and prediction intervals
% with a format that makes editing easy


[nfig]=PlotEllipses4DCA(fi,f_obs,f_obs_pred,FBoot,fj,lambda,Y,...
   nom_cat,ze_title,LesAxes,nfig,imp,gen_name,printype,printextension);
 
 
% Pretty Bootstrap + ConveHxull 
graph_boot=['Boot_',gen_name];
[nfig]=PlotPrettyBootStrap(fi,...
       lambda,nom_cat,FBoot,...
       f_obs,...
       ze_title,nfig,graph_boot,imp,LesAxes);
   

% Plot the rows + tolerance & prediction intervals   
graph_row=['Rows_',gen_name];
[nfig]=PlotRowBADA(lambda,fi,Y,f_obs,f_obs_pred,nom_cat,nom_r,...
ze_title,graph_row,nfig,imp,LesAxes);
 
 

% Plot the Blocks
% Create the names of the blocks before calling the plot routine
for k=1:nK;
    nom_tables{k}=['T=',int2str(k)];
    for i=1:nc; % nc is # of groups
     nom_cats{i,k}=[char(nom_cat{i}),'-',int2str(k)];
    end
end


[nfig]=PlotUglyBlocks(fi,fj,f_obs,lambda,BlockFactor,Tnum,...
      nom_cat,nom_cats,nom_r,nom_c,nom_tables,...
       LesAxes,ze_title,...
       nfig,imp,gen_name,printype,printextension);
   

disp(['Graphs done. All done!']);   
